#!/usr/local/bin/perl
# spurs.pl
#  -- probably more complicated than it has to be, but oh well. 
# -- no level prediction is done... too uncertain as far as I'm concerned
# 
# 
# Personal work of Lance Lascari.
# Copyright(C) 1999 Lance Lascari WS2B yadda yadda yadda
# lance_lascari@usa.net
# lance@rochester.rr.com
# http://home.rochester.rr.com/lascari
# LL 9/19/1999
# revised : 9/20/1999 LL -bug fix in indexing. & added FREL
# revised   11/10/1999 LL -cleaned up slightly
#
# Command line:  spurs inputfile.spi (output in outputfile.spo)
# and also in outputfile.csv

# first verify command line and file openings
$version="0.378 11/10/1999 LL";

die("USAGE: spurs [inputfile[.ext]] \r\n") 
    if( ($ARGV[0] eq "")  ) ;

unless( open(INDATA,$ARGV[0].".spi")) 
{   
    die("can't open input data file $ARGV[0].spi\r\n");
}

#check output file name length
$len=length($ARGV[0]);
if( $len > 8 )
{
    die("ERROR: I'll only allow you 8 characters in the output file name\n");
}
unless( open(OUTFILE,">".$ARGV[0].".spo"))     # append ".spo" extension
{
     die("can't open output file $ARGV[0].spo\r\n");
}

unless( open(OUTFILE2,">".$ARGV[0].".csv"))     # append ".csv" extension
{
     die("can't open output file $ARGV[0].csv\r\n");
}

# now start parsing the input data
printf OUTFILE ("Spur calculator output file from input file $ARGV[0].spi\n");
printf OUTFILE ("Version = [$version] (C) 1999 Lance Lascari lance_lascari\@usa.net\n");
#parse through the beginning comments, allow for up to 50 lines of them
printf OUTFILE ("Input file information\n");
$i=0;
$freq_begin=0;
$range_low=0;
$range_high=0;
$f_relative=0;
while($i<=50 && $freq_begin==0 )
{
    $i++;
    $linebuf=<INDATA>;                                # get one line of input  
    chop($linebuf);                                   #remove line feed
    printf OUTFILE ("%s\n",$linebuf);
    $firstchar=substr($linebuf,0,1);      # get first char
    if($firstchar eq "#")  # check for comment
    {

    }
    else # not comment, figure out what it is...
    {
# we'll be looking for the ORDER, SORTBY, and BEGFREQ words
# split the line into elements that were separated by whitespace
	@strbuf=split(/ /,$linebuf);  

	if(@strbuf[0] eq "ORDER")
	{
	    $order=@strbuf[1];
#	    printf ("Order is $order");
	}
	if(@strbuf[0] eq "DISPLAY")  #optional display range keyword
	{
	    $range_low=@strbuf[1];
	    $range_high=@strbuf[2];
	}

	if(@strbuf[0] eq "FREL")  #optional relative spur freq calc
	{
	    $f_relative=@strbuf[1];
	}
	if(@strbuf[0] eq "BEGFREQ")
	{
	    $freq_begin=1;  # this forces the while loop to drop out 
	}
    }
}
#end of basic header parsing

# freq parsing
$freq_index=-1;
while($freq_end==0 )
{
#
    $linebuf=<INDATA>;                                # get one line of input  
    chop($linebuf);                                   #remove line feed
    printf OUTFILE ("%s\n",$linebuf);
    $firstchar=substr($linebuf,0,1);      # get first char
    if($firstchar eq "#")  # check for comment
    {
	#ignore comment
    }
    else # not comment, figure out what it is...
    {  # assume it's NOT a frequency first and try to match the ENDFREQ keyword
# we'll be looking for the ORDER, SORTBY, and BEGFREQ words
# split the line into elements that were separated by whitespace
	@strbuf=split(/ /,$linebuf);  

	if(@strbuf[0] eq "ENDFREQ") #the end of the blcok
	{
	    $freq_end=1;
	}
	else # parse it assuming it's a valid frequency
	{
	    $freq_index++;  #count the number of frequencies starting at zero
	    @freq[$freq_index]=@strbuf[0]; # get frequency
	    @freq_desc[$freq_index]=@strbuf[1]; # get description
	    @freq_mod[$freq_index]=@strbuf[2]; # get modulatin bandwidth
	}
    }
}

#end freq parsing .... all input gathered

#for($i=0;$i<=$freq_index;$i++)
#{
#    printf OUTFILE ("freq [$i] = @freq[$i] desc [$i] = @freq_desc[$i] mod [$i] = @freq_mod[$i] \n");
#}


####################################################################
####################################################################
# now start computing stuff, start with harmonics as a separate case.
# output arrays, all with the same index (spindex), will be as follows
# spurout (spur frequency)
# spurdesc (spur description with numerical stuff)
# spurbw (spur bandwidth due to modulation)
# spurord (spur order)
####################################################################
####################################################################
$spindex=0;  # the master count of spurs


#straight harmonic calculations
for($i=0;$i<=$freq_index;$i++)  # once for each frequency
{
    for($j=1;$j<=$order;$j++) # once for each order
    {
	@spurout[$spindex]=$j*@freq[$i];  #individual spur
	@spurdesc[$spindex]="[$j x@freq_desc[$i]]";
	@spurbw[$spindex]=@freq_mod[$i]*$j;
	@spurord[$spindex]=$j;
	$spindex++;
    }
}
#end of straight harmonic calculations

#capture # of harmonic spurs generated
$harmonic_spurs=$spindex; 
#intermodulation computations
## use the arrays of data created by the harmonic calculations to reduce
## redundant calculations.
## this is totally brute-force and may need to be trimmed down.

for($i=0;$i<$harmonic_spurs;$i++)  # once for each harmonic spur
{
    for($j=0;$j<$harmonic_spurs;$j++) # once again for each harmonic spur
    {
	# freq, description, bandwidth, order
	#scan for output frequency range since sorting will go MUCH faster
	# if this array is decimated before hand
	
	$spurval=@spurout[$i]+@spurout[$j];
	
	if((($spurval >= $range_low) && ($spurval <= $range_high)) || ($range_high==0))
	{
	    
	    @spurout[$spindex]=$spurval;
	    @spurdesc[$spindex]="@spurdesc[$i] + @spurdesc[$j]";	
	    @spurbw[$spindex]=@spurbw[$i]+@spurbw[$j];
	    @spurord[$spindex]=@spurord[$i]+@spurord[$j];
	    $spindex++;
	}
	
	# check to see if the "minus" spur is negative and hence disqualified
	
	$spurval=@spurout[$i]-@spurout[$j];
	if($spurval>0)
	{
	    if((($spurval >= $range_low) && ($spurval <= $range_high)) || ($range_high==0))
	    {
		# freq, description, bandwidth, order
		@spurout[$spindex]=@spurout[$i]-@spurout[$j];
		@spurdesc[$spindex]="@spurdesc[$i] - @spurdesc[$j]";
		@spurord[$spindex]=@spurord[$i]+@spurord[$j];
		@spurbw[$spindex]=@spurbw[$i]+@spurbw[$j];  
		$spindex++; # increment counter since this was valid...
	    }
	}
    }
}


# end intermodulation computations




printf("$spindex spurs computed\n");

#@doohickey_states=("|","/","-","\\");

##### 
# try some basic sorting
#####
# bubble sort directly from "A book on C"

$sorting=1;

if($sorting==1)    
{
    printf("Sorting\n");
    
    for($i=0;$i<$spindex-1;++$i)
    {
	printf("%3d \% ",100*$i/($spindex-1));  #make the little spinning doo-hickey
	for($j=$spindex-1; $j>$i;--$j)
	{
	    if(@spurout[$j-1] > @spurout[$j])
	    {
		$ftemp=@spurout[$j-1];
		$desctemp=@spurdesc[$j-1];
		$ordtemp=@spurord[$j-1];
		$bwtemp=@spurbw[$j-1];
		
		@spurout[$j-1]=@spurout[$j];
		@spurdesc[$j-1]=@spurdesc[$j];
		@spurord[$j-1]=@spurord[$j];
		@spurbw[$j-1]=@spurbw[$j];
		
		@spurout[$j]=$ftemp;
		@spurdesc[$j]=$desctemp;
		@spurord[$j]=$ordtemp;
		@spurbw[$j]=$bwtemp;
	    }
	}
	printf("\b\b\b\b\b\b");
    }
}


# now try printing out results so far
printf OUTFILE ("results\n");
printf OUTFILE ("frequency description bandwidth order offset\n");
printf OUTFILE2 ("freq,order,offset,bw,desc\n");
$offset=0;
for($i=0;$i<$spindex;$i++)
{
    if(((@spurout[$i] >= $range_low) && (@spurout[$i] <= $range_high)) || ($range_high==0))
    {
	if($f_relative>0)
	{
	    $offset=@spurout[$i]-$f_relative;
	}
	
	printf OUTFILE ("%9.3f %30s %5.1f %3d %9.3f\n",@spurout[$i],@spurdesc[$i],@spurbw[$i],@spurord[$i],$offset);
	printf OUTFILE2 ("%9.3f,%3f,%9.3f,%5.1f, %s,\n",@spurout[$i],@spurord[$i],$offset,@spurbw[$i],@spurdesc[$i]);
    }
}




close(INDATA);
close(OUTFILE);
printf("\nThe End\n");
